/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class com_buildz_xf_LoaderJNI */

#ifndef _Included_com_buildz_xf_LoaderJNI
#define _Included_com_buildz_xf_LoaderJNI
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     com_buildz_xf_LoaderJNI
 * Method:    jloads
 * Signature: (Ljava/lang/String;)Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL Java_com_buildz_xf_LoaderJNI_jloads
  (JNIEnv *, jobject, jstring);

/*
 * Class:     com_buildz_xf_LoaderJNI
 * Method:    jloadx
 * Signature: (Ljava/lang/String;Z)Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL Java_com_buildz_xf_LoaderJNI_jloadx
  (JNIEnv *, jobject, jstring, jboolean);

#ifdef __cplusplus
}
#endif
#endif
